import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.lang.reflect.Method;


/**
 * A class representing the environment that a KarolBot moves around in.
 * 
 * @author Joel Ross, adapted from code by Zach Dodds and Wynn Vonnegut (HMC)
 * @version Sp15, v3
 */
public class Environment
{
    //directions
    public static final int NONE = -1;
    public static final int NORTH = 0;
    public static final int EAST = 1;
    public static final int SOUTH = 2;
    public static final int WEST = 3;
    
    //type constants
    private static final int CLEAR = 0; //required by maps
    private static final int WALL = 1; //required by maps
    private static final int ROBOT = 2;
    private static final int ERROR = 3;
    private static final int VISITED = 4;
    private static final int DONE = 5;
    private static final int COOKIEE = 6;
    private static final Color[] COLORS = new Color[] {Color.WHITE, Color.BLUE, Color.GREEN, Color.RED, Color.GRAY}; //matches above indices
    
    //maps -- values are the ordering
    private static final int EMPTY = 0;
    private static final int MAZE = 1;
    private static final int DIAMOND = 2;
    private static final int STALACTITE = 3;
    private static final int MAP4 = 4;
    private static final int MAP5 = 5;
    private static final int MAP6 = 6;
    private int[][] board; //the current board
    private int currentMap;

    //karolbot
    private Karol bot;
    private int botX, botY; //bot's position--it doesn't know. Stored here
    
    //gui elements
    private EnvironmentPanel canvas;
    private JButton goButton, stopButton, stepButton, resetButton, nextMapButton, prevMapButton, westButton, eastButton, northButton, southButton;
    private JTextField cellsField;
    private Timer clock;
    
    /**
     * Creates a new graphical environment for the Karol bot to explore.
     * Includes controlling buttons for moving the bot
     */
    public Environment()
    {
        JFrame frame = new JFrame("Karol");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setLayout(new BorderLayout());
        canvas = new EnvironmentPanel(); //for drawing the map
        frame.add(canvas, BorderLayout.CENTER);
        
        ControlListener cl = new ControlListener();

        JPanel botControl = new JPanel();
        botControl.setLayout(new FlowLayout());
        goButton = new JButton("Start Karol"); goButton.addActionListener(cl);
        stopButton = new JButton("Stop Karol"); stopButton.addActionListener(cl);
        stepButton = new JButton("Step Karol"); stepButton.addActionListener(cl);
        resetButton = new JButton("Reset"); resetButton.addActionListener(cl);
        botControl.add(goButton);
        botControl.add(stopButton);
        botControl.add(stepButton);
        botControl.add(resetButton);

        clock = new Timer(20, cl);
        
        JPanel mapControl = new JPanel();
        mapControl.setLayout(new FlowLayout());
        mapControl.add(new JLabel("Cells to go:"));
        cellsField = new JTextField(5);
        cellsField.setEditable(false);
        mapControl.add(cellsField);
        nextMapButton = new JButton("Next Map ->"); nextMapButton.addActionListener(cl);
        prevMapButton = new JButton("<- Prev Map"); prevMapButton.addActionListener(cl);
        mapControl.add(prevMapButton);
        mapControl.add(nextMapButton);

        JPanel teleControl = new JPanel();
        teleControl.setLayout(new FlowLayout());
        northButton = new JButton("North"); northButton.addActionListener(cl);
        southButton = new JButton("South"); southButton.addActionListener(cl);
        eastButton = new JButton("East"); eastButton.addActionListener(cl);
        westButton = new JButton("West"); westButton.addActionListener(cl);
        teleControl.add(new JLabel("Teleport Robot:"));
        teleControl.add(northButton);
        teleControl.add(eastButton);
        teleControl.add(southButton);
        teleControl.add(westButton);
        
        JPanel controls = new JPanel();
        controls.setLayout(new GridLayout(3,1));
        controls.add(botControl);
        controls.add(mapControl);
        controls.add(teleControl);
        frame.add(controls, BorderLayout.SOUTH);
        
        selectMap(EMPTY);
        reset(); //position bot on the map before showing. Also resets the gui/map values
        
        //supporting ultra-high resolution screens
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        }catch(Exception e){}
        
        frame.pack();
        frame.setVisible(true);
    }

    /**
     * Deploys the given Karol bot to the environment, so that it can do the exploring
     * @param bot The Karol bot to place in the environment
     */
    public void deploy(Karol bot)
    {
        this.bot = bot;
        reset();
    }
        
    //has the bot take a step, moving direction indicated by map-appropriate method
    private void stepBot()
    {
        if(bot == null) return;

        //do some logic based on the map to determine the direction!
        if(currentMap == EMPTY)
            moveBot(bot.stepEmptyMap());
        else if(currentMap == DIAMOND)
            moveBot(bot.stepDiamondMap());
        else if(currentMap == MAZE)
            moveBot(bot.stepMazeMap());
        else if(currentMap == STALACTITE)
            moveBot(bot.stepStalactiteMap());
        else if(currentMap == MAP4)
            moveBot(bot.stepMap4());
        else if(currentMap == MAP5)
            moveBot(bot.stepMap5());
        else if(currentMap == MAP6)
            moveBot(bot.stepMap6());
    }
        
    //move the bot in specified direction. Don't go out of bounds or into a wall.
    private void moveBot(int dir)
    {
        int nx = botX;
        int ny = botY;
        if(dir == NORTH && !hasWallNorth())
                ny = botY-1;
        else if(dir == SOUTH && !hasWallSouth())
                ny = botY+1;        
        else if(dir == EAST && !hasWallEast())
                nx = botX+1;                    
        else if(dir == WEST && !hasWallWest())
                nx = botX-1;            
        
        //actually move
        board[botX][botY] = VISITED;
        botX = nx;
        botY = ny;
        board[botX][botY] = ROBOT;
        
        updateGUI();
    }

    //teleports bot in specified direction. Will not go into walls or out of bounds.
    //Prints message if cannot move
    private void teleportBot(int dir)
    {
        clock.stop(); //stop moving

        int nx = botX;
        int ny = botY;
        selectMap(currentMap); //clear the map
        
        if(dir == NORTH) {
            if(!hasWallNorth())
                ny = botY-1;
            else
                System.out.println("There is a wall to the north. Not moving the robot.");
        }
        else if(dir == SOUTH) {
            if(!hasWallSouth())
                ny = botY+1;
            else
                System.out.println("There is a wall to the south. Not moving the robot.");
        }
        else if(dir == EAST) {
            if(!hasWallEast())
                nx = botX+1;
            else
                System.out.println("There is a wall to the east. Not moving the robot.");               
        }
        else if(dir == WEST) {
            if(!hasWallWest())
                nx = botX-1;
            else
                System.out.println("There is a wall to the west. Not moving the robot.");                               
        }
        
        //board[botX][botY] = CLEAR;
        botX = nx;
        botY = ny;
        board[botX][botY] = ROBOT;

        updateGUI();
    }   
    
    //resets the robot (to random location)
    private void reset()
    {
        clock.stop(); //stop moving

        if(this.bot != null)
            this.bot.setState(0);
        ArrayList<Point> options = new ArrayList<Point>();
        for(int i=0; i<board.length; i++) {
            for(int j=0; j<board[i].length; j++){
                if(board[i][j] != WALL)
                {
                    options.add(new Point(i,j));
                    board[i][j] = CLEAR; //clears old bots and visited and everything
                }
            }
        }
        Point p = options.get((int)(Math.random()*options.size()));
        botX = p.x;
        botY = p.y;
        board[botX][botY] = ROBOT;

        updateGUI();
    }

    
    
    /**
     * Returns whether there is a wall to the north of the Karol bot
     */
    public boolean hasWallNorth()
    {
        return botY != 0 && board[botX][botY-1] == WALL;
    }
    
    /**
     * Returns whether there is a wall to the south of the Karol bot
     */
    public boolean hasWallSouth()
    {
        return botY != board.length-1 && board[botX][botY+1] == WALL;
    }

    /**
     * Returns whether there is a wall to the east of the Karol bot
     */
    public boolean hasWallEast()
    {
        return botX != board.length-1 && board[botX+1][botY] == WALL;
    }
    
    /**
     * Returns whether there is a wall to the west of the Karol bot
     */
    public boolean hasWallWest()
    {
        return botX != 0 && board[botX-1][botY] == WALL;
    }
    
    /**
     * Returns the Karol bot's current surroundings in String format.
     * Surroundings are listed in NEWS order, with the capital letter when there IS a wall,
     * and an 'x' when there IS NOT a wall.
     * 
     * So NxWx represents walls to the North and West, but not to East and South.
     * 
     * @return A String representing the current surroundings.
     */
    public String getSurroundings()
    {
        String toReturn = "";
        if(hasWallNorth()) 
            toReturn += "N";
        else 
            toReturn += "x";
        if(hasWallEast()) 
            toReturn += "E";
        else 
            toReturn += "x";
        if(hasWallWest()) 
            toReturn += "W";
        else 
            toReturn += "x";
        if(hasWallSouth()) 
            toReturn += "S";
        else 
            toReturn += "x";
        return toReturn;
    }
    
    /*******
     * GUI *
     *******/
    
    //inner class to handle the graphical drawing
    private class EnvironmentPanel extends JPanel
    {
        private final int SIZE;
        public EnvironmentPanel() {
           //handle multiple resolutions
           Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
           if(screenSize.height < 1000) //tiny projector
               SIZE = 300;
           else if(screenSize.width < 2000) //normal screen
               SIZE = 500;
           else //hi-res display
               SIZE = 1000;    

           this.setPreferredSize(new Dimension(SIZE,SIZE));
        }
        
        public void paintComponent(Graphics g)
        {
            g.setColor(COLORS[CLEAR]);
            g.fillRect(0, 0, SIZE, SIZE);
            
            int len = SIZE/board.length; //should be 20
            
            //draw the map
            for(int i=0; i<board.length; i++) {
                for(int j=0; j<board[i].length; j++)
                {
                    g.setColor(COLORS[board[i][j]]);
                    g.fillRect(i*len, j*len, len, len);
                }
            }

            //draw the Robot. More fun? Can make available?
            //g.setColor(COLORS[ROBOT]);
            //g.fillRect(botX*len, botY*len, len, len);
        
            try{
                Method drawMethod = Karol.class.getMethod("draw", new Class[] { Graphics.class });
                if(drawMethod != null)
                {
                    g.setColor(Color.WHITE);
                    g.translate(botX*len, botY*len);
                    g.fillRect(0,0, len, len);
                    drawMethod.invoke(bot, g);
                    g.translate(-botX*len, -botY*len);
                }
            }catch (Exception e){}
            
        }
    }
    
    //inner class for handling button actions, so avoid making actionPerformed publicly available
    private class ControlListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
            if(e.getSource() == clock)
                stepBot();
            else if(e.getSource() == stepButton)
                stepBot();
            else if(e.getSource() == goButton)
                clock.restart();
            else if(e.getSource() == stopButton)
                clock.stop();
            else if(e.getSource() == resetButton)
                reset();
            else if(e.getSource() == nextMapButton){
                selectMap((currentMap+1)%7);
                reset();
            }
            else if(e.getSource() == prevMapButton){
                selectMap((currentMap+6)%7);
                reset();
            }
            else if(e.getSource() == northButton)
                teleportBot(NORTH);
            else if(e.getSource() == eastButton)
                teleportBot(EAST);
            else if(e.getSource() == southButton)
                teleportBot(SOUTH);
            else if(e.getSource() == westButton)
                teleportBot(WEST);
            
        }
    }
    
    //refreshes the gui based on the current state of the environment. Includes an alert if the map is cleared
    private void updateGUI()
    {
        canvas.repaint();
        int cellsLeft = countSquares(CLEAR);
        cellsField.setText(""+countSquares(CLEAR));

        if(cellsLeft == 0){
            clock.stop();
            System.out.println("Complete!");
            JOptionPane.showMessageDialog(null, "Map completed!", "Complete!", JOptionPane.PLAIN_MESSAGE);
        }

        //update any other GUI elements
    }   

    //loads in the given map number. This acts as a mapping between map names and contents
    private void selectMap(int map)
    {
        currentMap = map;
        if(currentMap == EMPTY)
            board = emptyMap();
        if(currentMap == DIAMOND)
            board = diamondMap();
        if(currentMap == MAZE)
            board = mazeMap();
        if(currentMap == STALACTITE)
            board = stalactiteMap();
        if(currentMap == MAP4)
            board = map4();
        if(currentMap == MAP5)
            board = map5();
        if(currentMap == MAP6)
            board = map6();
        updateGUI();
    }
    
    //returns the number of a particular kind of square in the map. Used for counting visited cells
    private int countSquares(int type)
    {
        int count = 0;
        for(int i=0; i<board.length; i++) {
            for(int j=0; j<board[i].length; j++){
                if(board[i][j] == type)
                    count++;
            }
        }
        return count;
    }

    
    
    /********
     * MAPS *
     ********/
    
    private static int[][] emptyMap()
    {
        return new int[][] {
           { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 }
        };
    }
    
    
//  private static int[][] almostFullMap()
//  {
//      return new int[][] {
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
//          { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 } 
//      };
//  }
    
    private static int[][] diamondMap()
    {
        return new int[][] {
           { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
           { 1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1 },
           { 1,1,1,1,1,1,1,1,1,1,1,0,0,0,1,1,1,1,1,1,1,1,1,1,1 },
           { 1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1 },
           { 1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1 },
           { 1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1 },
           { 1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1 },
           { 1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1 },
           { 1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1 },
           { 1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1 },
           { 1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1 },
           { 1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1 },
           { 1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1 },
           { 1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1 },
           { 1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1 },
           { 1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1 },
           { 1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1 },
           { 1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1 },
           { 1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1 },
           { 1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1 },
           { 1,1,1,1,1,1,1,1,1,1,1,0,0,0,1,1,1,1,1,1,1,1,1,1,1 },
           { 1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1 },
           { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 }                
        };
    }
    
    private static int[][] mazeMap()
    {
        return new int[][] {
           { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
           { 1,0,0,1,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,1,0,1 },
           { 1,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,0,0,1,0,0,0,0,1 },
           { 1,1,0,1,0,1,1,1,0,1,0,1,0,1,0,1,1,1,1,1,1,1,1,0,1 },
           { 1,0,0,1,0,0,0,0,0,1,0,1,0,1,0,0,0,1,0,0,0,0,1,0,1 },
           { 1,0,1,1,1,0,1,1,1,1,0,1,0,1,0,1,0,1,0,1,1,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,1,0,1,0,1,0,1,0,1,1,1,1,1,1 },
           { 1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,0,1,0,0,1,0,0,0,1 },
           { 1,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,1,1,0,1,0,1,1,1 },
           { 1,1,0,1,0,1,0,0,0,1,0,1,0,1,1,1,0,0,1,0,1,0,0,0,1 },
           { 1,1,0,1,0,1,1,1,1,1,0,1,0,1,0,1,0,1,1,0,1,0,1,0,1 },
           { 1,1,0,1,0,0,0,0,0,1,0,0,0,1,0,0,0,0,1,0,1,1,1,0,1 },
           { 1,1,0,1,1,1,1,1,0,1,1,0,1,1,1,1,1,0,1,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,1,0,1,1,0,0,0,0,0,1,0,1,1,1,0,1,0,1 },
           { 1,0,1,1,1,1,1,1,0,1,0,0,1,1,1,0,1,0,0,0,1,0,1,0,1 },
           { 1,0,0,0,0,0,0,0,0,1,0,1,1,0,1,0,1,1,1,0,1,0,1,0,1 },
           { 1,1,1,1,1,1,1,1,1,1,0,1,0,0,1,0,0,0,1,0,1,0,1,0,1 },
           { 1,0,0,0,0,1,0,0,0,1,1,1,0,1,1,1,1,0,1,0,1,0,1,0,1 },
           { 1,0,1,1,0,1,0,1,0,0,0,0,0,0,0,0,1,0,1,0,1,1,1,0,1 },
           { 1,0,0,1,0,1,0,1,1,1,1,1,0,1,1,0,1,0,1,0,1,0,1,0,1 },
           { 1,0,1,1,0,1,0,1,0,1,0,0,0,1,0,0,1,0,0,0,1,0,0,0,1 },
           { 1,0,1,0,0,1,0,1,0,0,0,1,1,1,0,1,1,1,1,1,1,1,0,1,1 },
           { 1,0,1,1,1,1,0,1,0,1,0,0,0,1,0,1,0,0,0,0,0,1,0,1,1 },
           { 1,0,0,0,0,0,0,1,0,1,1,1,0,1,0,0,0,1,1,1,0,0,0,1,1 },
           { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 }
        };
    }

    private static int[][] stalactiteMap()
    {
        return new int[][] {
           { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
           { 1,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,1,1,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1 },
           { 1,0,0,0,1,1,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1 },
           { 1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1 },
           { 1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1 },
           { 1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1 },
           { 1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,1 },
           { 1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1 },
           { 1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1 },
           { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 }
        };
    }

    private static int[][] map4()
    {
        return new int[][] {
           { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
           { 1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,1 },
           { 1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,1 },
           { 1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,1,0,0,1,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,1 },
           { 1,0,0,0,1,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,1 },
           { 1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,1 },
           { 1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,1 },
           { 1,0,1,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,1,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,1,1,1,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,1 },
           { 1,0,1,1,0,0,1,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,1,1 },
           { 1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1 },
           { 1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,1 },
           { 1,0,0,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,1,1,1,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,1,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,1,1,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1 },
           { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 }
        };
    }

    private static int[][] map5()
    {
        return new int[][] {
           { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,1 },
           { 1,0,0,0,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,1,1,0,0,0,1 },
           { 1,0,0,0,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,1 },
           { 1,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,1 },
           { 1,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,1 },
           { 1,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 }
        };
    }

    private static int[][] map6()
    {
        return new int[][] {
           { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,0,1 },
           { 1,1,0,0,0,0,1,0,0,1,0,0,0,0,0,0,1,1,0,0,0,1,0,0,1 },
           { 1,0,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,0,1,0,0,0,1 },
           { 1,0,1,1,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1,0,0,0,0,1 },
           { 1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,1,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,1,0,0,0,0,1,0,1,0,1,1,1,1,0,0,0,1,0,1 },
           { 1,0,0,0,0,1,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,1,1 },
           { 1,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,1,0,0,0,0,0,0,0,0,1,0,0,0,0,1,0,0,0,0,0,0,0,1 },
           { 1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,1 },
           { 1,0,1,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1 },
           { 1,1,0,0,1,1,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,0,0,0,1 },
           { 1,0,0,0,0,1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1 },
           { 1,0,0,0,0,1,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,1,0,0,1 },
           { 1,0,0,0,0,0,1,0,0,0,1,0,0,0,1,1,1,0,0,0,0,1,1,0,1 },
           { 1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,1,1,1,0,0,0,0,0,1 },
           { 1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,1 },
           { 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,1 },
           { 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1 }
        };
    }
    
}
